#!/usr/bin/env php
<?php
    use AdamBrett\ShellWrapper\Runners\Exec;
    use AdamBrett\ShellWrapper\Command\Builder as CommandBuilder;
    require 'vendor/autoload.php';
    require 'colors.class.php';
    $installedFail2Ban = $installedIPTables = false;
    $osDistribution = $osDistributionName = $osDistributionVersion = null;
    $configData = readClipsConfig();
    if (!isset($configData->serviceUrl))
        $configData->serviceUrl = 'https://clips.saas-secure.com';
    if (!isset($configDate->selfProtect))
        $configData->selfProtect = true;
    if (!isset($configDate->debuglevel))
        $configData->debuglevel = 3;
    if (!isset($configData->listenIP))
        $configData->listenIP = '0.0.0.0';
    echo "\n";
    $cliColor = new colors();
    // check user permission at system


    // OS info
    echo 'Operating System is: ' . php_uname('s') . ' - ' . php_uname('v') . "\n";
    // check distro
    $shell = new Exec();
    $command = new CommandBuilder('./setup/distro.sh');
    $shell->run($command);
    $output = $shell->getOutput();
    $osDistribution = implode(";", $output);
    $osDistributionVersion = trim(preg_replace('=[a-zA-Z]+=', '', $osDistribution));
    $osDistributionName = trim(preg_replace('=\s.+$=', '', str_replace($osDistributionVersion, '', $osDistribution)));
    echo 'Distribution: ' . $osDistributionName . ' Version: ' . $osDistributionVersion . "\n";

    // check if iptables is installed
    $shell = new Exec();
    $command = new CommandBuilder('command -v iptables');
    $shell->run($command);
    #echo implode("\n", $shell->getOutput()) . "\n";
    if ($shell->getReturnValue() == '0')
        $installedIPTables = true;

    // check if fail2ban is installed
    $shell = new Exec();
    $command = new CommandBuilder('command -v fail2ban-client');
    $shell->run($command);
    #echo implode("\n", $shell->getOutput()) . "\n";
    #echo $shell->getReturnValue() . '---';
    if ($shell->getReturnValue() == '0')
        $installedFail2Ban = true;

    if (!$installedIPTables)
        if (!distriInstall('iptables', $osDistributionName))
        {
            echo $cliColor->getColoredString('IPTables installation failed', 'red');
            echo "\nPlease try to install manually and restart CLIPS installer";
            exit(1);
        }
    if ($installedFail2Ban)
    {
        // check version
        $shell = new Exec();
        $command = new CommandBuilder('fail2ban-client -V');
        $shell->run($command);
        if ($shell->getReturnValue() != '0')
        {
            echo $cliColor->getColoredString('Required application location failed', 'red');
            echo "\nPlease contact support, see: http://www.saas-secure.com, to solve this problem";
            exit(1);
        }
        // fetch version
        $tmp = $shell->getOutput();
        $fail2banVersion = '0.0.0';
        foreach ($tmp as $line)
        {
            if (preg_match('=fail2ban v.+$=i', $line))
            {
                $fail2banVersion = trim(preg_replace('=fail2ban v=i', '', $line));
                break;
            }
        }
        #echo 'Fail2Ban Version: ' . $fail2banVersion . "\n";
        if (version_compare($fail2banVersion, '0.9.3', '<')) {
            // php version isn't high enough
            echo $cliColor->getColoredString('Fail2Ban version is too old!', 'red');
            echo "\nPlease install at least Fail2Ban Version 0.9.3\n";
            exit(1);
        }
    }

    // check php version
    echo 'Installed PHP Version is: ' . phpversion() . ' ';

    if (version_compare(phpversion(), '5.3.0', '<')) {
        // php version isn't high enough
        echo $cliColor->getColoredString('this version is too old!', 'red');
        echo "\nPlease install at least PHP Version 5.3";
        exit(1);
    }
    else
        echo  $cliColor->getColoredString("okay", 'green');
    echo "\n";

    // php.ini loaded
    echo "php.ini loaded: " . (php_ini_loaded_file() ? php_ini_loaded_file() : $cliColor->getColoredString('none', 'yellow')) . "\n";

    // check php soap is available
    echo 'PHP SOAP is enabled: ';
    if (!get_loaded_extensions('soap')) {
        // php version isn't high enough
        echo $cliColor->getColoredString('not enabled!', 'red');
        echo "\nPlease enable PHP SOAP extension";
    }
    else
        echo  $cliColor->getColoredString("enabled", 'green');
    echo "\n";
    // check if sockets are supported
    echo 'PHP SOCKETS is enabled: ';
    if (!get_loaded_extensions('sockets')) {
        // php version isn't high enough
        echo $cliColor->getColoredString('not enabled!', 'red');
        echo "\nPlease enable PHP SOCKETS extension";
    }
    else
        echo  $cliColor->getColoredString("enabled", 'green');
    echo "\n";

    // stop service
    if (file_exists('/etc/init.d/clips'))
    {
        $shell = new Exec();
        $command = new CommandBuilder('/etc/init.d/clips stop');
        $shell->run($command);
    }

    // ask for port
    $line = null;
    while (!is_integer($line))
    {
        $line = trim(readline('Port (default=933): '));
        if ($line == '')
            $line = 933;
        if (!preg_match('=^[0-9]+$=', $line))
            echo $cliColor->getColoredString('Numbers only!', 'red') . "\n";
        else
        {
            $line = (int)$line;
            // check if port is used
            $shell = new Exec();
            $command = new CommandBuilder('netstat -ln | grep \':' . $line . ' \' | grep \'LISTEN\'');
            $shell->run($command);
            #echo implode("\n", $shell->getOutput()) . "\n";
            if($shell->getOutput() || $shell->getReturnValue() == 0)
            {
                $line = null;
                echo $cliColor->getColoredString('Select another port, because is already in use!', 'red') . "\n";
            }
        }
    }

    $port = $line;
    $configData->port = $port;
    writeClipsConfig($configData);

    // ask for API key
    $line = null;
    while (!$line)
    {
        if (isset($configData->apikey) && $configData->apikey)
            $line = trim(readline('CLIPS API-Key (preset=' . $configData->apikey . '): '));
        else
            $line = trim(readline('CLIPS API-Key (Get at: https://clips.saas-secure.com): '));
        if ($line == '' && isset($configData->apikey) && $configData->apikey)
            $line = $configData->apikey;
    }
    $configData->apikey = $line;
    writeClipsConfig($configData);

    createServerPem();

    // get latest phar
    $xmlSource = file_get_contents($configData->serviceUrl . "/version.xml");
    try
    {
        $xml = new SimpleXMLElement($xmlSource);
    }
    catch (Exception $e)
    {
        throw new Exception ('Update XML unreadable', '00');
    }
    #echo var_dump($xml);
    if (!$xml || !$xml->latest->version)
    {
        echo $cliColor->getColoredString('CLIPS daemon script download failed!', 'red');
        echo "\n\n";
        exit(1);
    }
    $sourceFile = @file_get_contents($xml->latest->url);
    if (!$sourceFile)
    {
        echo $cliColor->getColoredString('CLIPS daemon script download not possible!', 'red');
        echo "\n\n";
        exit(1);
    }
    // copy daemon script
    switch (strtolower($osDistributionName))
    {
        case 'ubuntu':
            #continue;
            $shell = new Exec();
            $command = new CommandBuilder('cp ' . dirname(__FILE__) . '/clips-daemon.php /usr/bin && chmod +x /usr/bin/clips-daemon.php');
            $shell->run($command);
            #echo implode("\n", $shell->getOutput()) . "\n";
            if ($shell->getReturnValue() != '0')
            {
                echo $cliColor->getColoredString('CLIPS daemon script copy failed!', 'red');
                echo "\n\n";
                exit(1);
            }
            // write file
            if (!$fp = @fopen('/var/lib/clips/clips-daemon.phar.gz', 'w'))
            {
                echo $cliColor->getColoredString('Daemon executable not writable!', 'red');
                echo "\n\n";
                exit(1);
            }
            @fwrite($fp, $sourceFile);
            fclose($fp);
            $shell = new Exec();
            $command = new CommandBuilder('chmod 0600 /var/lib/clips/clips-daemon.phar.gz');
            $shell->run($command);
            #echo implode("\n", $shell->getOutput()) . "\n";
            if ($shell->getReturnValue() != '0')
            {
                echo $cliColor->getColoredString('CLIPS daemon script copy failed!', 'red');
                echo "\n\n";
                exit(1);
            }
            break;
        default:
            return false;
            break;
    }

    // copy startup script
    switch (strtolower($osDistributionName))
    {
        case 'ubuntu':
            $shell = new Exec();
            $command = new CommandBuilder('cp ' . dirname(__FILE__) . '/clips /etc/init.d && chmod +x /etc/init.d/clips');
            $shell->run($command);
            #echo implode("\n", $shell->getOutput()) . "\n";
            if ($shell->getReturnValue() != '0')
            {
                echo $cliColor->getColoredString('CLIPS startup script copy failed!', 'red');
                echo "\n\n";
                exit(1);
            }
            // run level
            $rl = array ('/etc/rc2.d', '/etc/rc3.d', '/etc/rc4.d', '/etc/rc5.d');
            foreach ($rl as $path)
            {
                $shell = new Exec();
                $command = new CommandBuilder('ln -sf /etc/init.d/clips ' . $path . '/S06clips');
                $shell->run($command);
                #echo implode("\n", $shell->getOutput()) . "\n";
                if ($shell->getReturnValue() != '0')
                {
                    echo $cliColor->getColoredString('CLIPS startup script link failed!', 'red');
                    echo "\n\n";
                    exit(1);
                }
            }
            break;
        default:
            return false;
            break;
    }



    // copy iptables clips
    $shell = new Exec();
    $command = new CommandBuilder('cp ' . dirname(__FILE__) . '/iptables-clips.conf /etc/fail2ban/action.d && chmod a-x /etc/fail2ban/action.d/iptables-clips.conf');
    $shell->run($command);
    #echo implode("\n", $shell->getOutput()) . "\n";
    if ($shell->getReturnValue() != '0')
    {
        echo $cliColor->getColoredString('CLIPS jail action copy failed!', 'red');
        echo "\n\n";
        exit(1);
    }
    // check if jail.local exists
    if (file_exists('/etc/fail2ban/jail.local'))
    {
        // backup
        $shell = new Exec();
        $command = new CommandBuilder('cp /etc/fail2ban/jail.local /etc/fail2ban/jail.local.bak');
        $shell->run($command);
        #echo implode("\n", $shell->getOutput()) . "\n";
        if ($shell->getReturnValue() != '0')
        {
            echo $cliColor->getColoredString('CLIPS jail.local backup failed!', 'red');
            echo "\n\n";
            exit(1);
        }
    }
    else
    {
        // copy from source
        $shell = new Exec();
        $command = new CommandBuilder('cp /etc/fail2ban/jail.conf /etc/fail2ban/jail.local');
        $shell->run($command);
        #echo implode("\n", $shell->getOutput()) . "\n";
        if ($shell->getReturnValue() != '0')
        {
            echo $cliColor->getColoredString('CLIPS jail.local copy failed!', 'red');
            echo "\n\n";
            exit(1);
        }
    }


    $file = '/etc/fail2ban/jail.local';
    $tmp = file_get_contents($file);
    if (!$tmp)
    {
        echo $cliColor->getColoredString('jail.local not readable!', 'red');
        echo "\n\n";
        exit(1);
    }
    $bantime = 259200;
    $maxretry = 3;
    if (!preg_match("/\nbantime = " . $bantime . "\n" . '/siU', $tmp))
        $tmp = preg_replace("/\nbantime\s+= [0-9]+\n/siU", "\nbantime = $bantime\n", $tmp);
    if (!preg_match("/\nmaxretry = " . $maxretry . "\n" . '/siU', $tmp))
        $tmp = preg_replace("/\nmaxretry\s= [0-9]+\n/siU", "\nmaxretry = $maxretry\n", $tmp);
    $tmp = preg_replace("/\naction\s+=\s+%\(action_mwl?\)s\n/siU", "\naction = %(action_)s\n", $tmp);
    $tmp = preg_replace("/\nbanaction\s+=\s+.+\n/siU", "\nbanaction = iptables-clips\n", $tmp);
    $tmp = preg_replace("/\n\[blocklist\].+\n#\[blocklist\]\n/siU", "\n", $tmp);
    $tmp .= "\n"
        . "[blocklist]\n"
        . "enabled  =  true\n"
        . "port     =  ssh\n"
        . "filter   =  sshd\n"
        . "logpath  =  /etc/fail2ban/empty.log\n"
        . "maxretry =  1\n"
        . "banaction = iptables-allports\n"
        . "action   = %(action_)s\n"
        . "#[blocklist]\n";
    // write back
    if (!$fp = @fopen($file, 'w+'))
    {
        echo $cliColor->getColoredString('jail.local not writeable!', 'red');
        echo "\n\n";
        exit(1);
    }
    else
    {
        @fwrite($fp, $tmp);
        fclose($fp);
    }

    $shell = new Exec();
    $command = new CommandBuilder('touch /etc/fail2ban/empty.log');
    $shell->run($command);

    // restart fail2ban

    // start service
    $shell = new Exec();
    $command = new CommandBuilder('/etc/init.d/clips start');
    $shell->run($command);

    echo $cliColor->getColoredString('CLIPS setup successful done!', 'green');
    echo "\n\n";


function distriInstall($package, $distribution)
{
    switch (strtolower($distribution))
    {
        case 'ubuntu':
            $shell = new Exec();
            $command = new CommandBuilder('apt-get --assume-yes -qq install ' . $package . ' > /dev/null');
            $shell->run($command);
            #echo implode("\n", $shell->getOutput()) . "\n";
            if ($shell->getReturnValue() == '0')
                return true;
            break;
        default:
            return false;
            break;
    }
    return false;
}
function readClipsConfig()
{
    $clipsConfPath = '/etc/clips.conf';
    if (!file_exists($clipsConfPath))
        return new stdClass();
    $tmp = file_get_contents($clipsConfPath);
    return json_decode($tmp);
}
function writeClipsConfig($data)
{
    $clipsConfPath = '/etc/clips.conf';
    $cliColor = new colors();
    if (!$fp = @fopen($clipsConfPath, 'w'))
    {
        echo $cliColor->getColoredString('clips.conf not writeable!', 'red');
        echo "\n\n";
        exit(1);
    }
    else
    {
        @fwrite($fp, json_encode($data));
        fclose($fp);
    }
    return true;
}
function createServerPem()
{
    $cliColor = new colors();
    if (!is_dir('/var/lib/clips'))
        if (!mkdir('/var/lib/clips', 0600))
        {
            echo $cliColor->getColoredString('/var/lib/clips not created!', 'red');
            echo "\n\n";
            exit(1);
        }
    $certificateData = array(
        "countryName" => "DE",
        "stateOrProvinceName" => "Saxony",
        "localityName" => "Leipzig",
        "organizationName" => "S.Ruttloff, Computer Hard- & Software",
        "organizationalUnitName" => "Cloud IPS",
        "commonName" => "SaaS-Secure.com",
        "emailAddress" => "info@saas-secure.com"
    );

    // Generate certificate
    $privateKey = openssl_pkey_new();
    $certificate = openssl_csr_new($certificateData, $privateKey);
    $certificate = openssl_csr_sign($certificate, null, $privateKey, (5*365));

    $configData = readClipsConfig();
    // Generate PEM file
    $pem_passphrase = $configData->apikey;
    $pem = array();
    openssl_x509_export($certificate, $pem[0]);
    openssl_pkey_export($privateKey, $pem[1], $pem_passphrase);
    $pem = implode($pem);

    // Save PEM file
    $pemfile = '/var/lib/clips/server.pem';
    if (!file_put_contents($pemfile, $pem))
    {
        echo $cliColor->getColoredString('PEM file not created!', 'red');
        echo "\n\n";
        exit(1);
    }
    $configData->pemValidTo = date('Y-m-d H:i:s', strtotime('+' . (5*365) . ' Days'));
    writeClipsConfig($configData);
}
